<?php

namespace Drupal\commerce_dibs;

use Drupal\commerce_dibs\Event\BuildOrderEvent;
use Drupal\commerce_dibs\Event\DibsEvents;
use Drupal\commerce_order\Adjustment;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_order\Entity\OrderItemInterface;
use Drupal\commerce_product\Entity\ProductVariation;
use Drupal\commerce_product\Entity\ProductVariationInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;

class EasyOrderBuilder {

  use StringTranslationTrait;

  /**
   * @var \Symfony\Component\EventDispatcher\EventDispatcher
   */
  protected $eventDispatcher;

  public function __construct(EventDispatcherInterface $eventDispatcher) {
    $this->eventDispatcher = $eventDispatcher;
  }

  /**
   * @param \Drupal\commerce_order\Entity\OrderInterface $order
   *
   * @return array
   */
  public function buildOrder(OrderInterface $order) {
    $dibs_order = [
      'order' => [
        'items' => [],
        'amount' => $order->getTotalPrice()->multiply(100)->getNumber(),
        'currency' => $order->getTotalPrice()->getCurrencyCode(),
        'reference' => $order->id(),
      ],
      'checkout' => [
        'termsUrl' => Url::fromRoute('<front>')->setAbsolute()->toString(),
      ],
    ];

    if ($shipments = $order->get('shipments')) {
      /** @var \Drupal\commerce_shipping\Entity\ShipmentInterface $shipment */
      $shipment = $shipments->first()->entity;
      $profile = $shipment->getShippingProfile();
      $dibs_order['checkout']['merchantHandlesConsumerData'] = TRUE;
      $dibs_order['checkout']['consumer']['reference'] = $profile->id();
      $dibs_order['checkout']['consumer']['email'] = $order->getEmail();
      $dibs_order['checkout']['consumer']['shippingAddress']['addressLine1'] = $profile->address->address_line1;
      $dibs_order['checkout']['consumer']['shippingAddress']['addressLine2'] = $profile->address->address_line2;
      $dibs_order['checkout']['consumer']['shippingAddress']['postalCode'] = $profile->address->postal_code;
      $dibs_order['checkout']['consumer']['shippingAddress']['city'] = $profile->address->locality;
      /** @var \CommerceGuys\Addressing\Country\CountryRepository $country_repository */
      $country_repository = \Drupal::service('address.country_repository');
      $country = $country_repository->get($profile->address->country_code);
      $dibs_order['checkout']['consumer']['shippingAddress']['country'] = $country->getThreeLetterCode();
      $dibs_order['checkout']['consumer']['privatePerson']['firstName'] = $profile->address->given_name;
      $dibs_order['checkout']['consumer']['privatePerson']['lastName'] = $profile->address->family_name;
    }

    foreach ($order->getItems() as $item) {
      $dibs_order['order']['items'][] = $this->buildItem($item);
    }
    foreach ($order->getAdjustments() as $adjustment) {
      $dibs_order['order']['items'][] = $this->buildAdjustment($adjustment);
    }

    $event = new BuildOrderEvent($order, $dibs_order);
    $this->eventDispatcher->dispatch(DibsEvents::EASY_BUILD_ORDER, $event);
    return $event->getDibsOrder();
  }

  /**
   * @param \Drupal\commerce_order\Entity\OrderItemInterface $item
   *
   * @return array
   */
  public function buildItem(OrderItemInterface $item) {
    $dibsItem = [
      'reference' => (($variation = $item->getPurchasedEntity()) instanceof ProductVariationInterface) ? $variation->getSku() : $item->id(),
      'name' => $item->label(),
      'quantity' => $item->getQuantity(),
      'unit' => $this->t('Unit'),
      'taxRate' => 0,
      'taxAmount' => 0,
      'unitPrice' => $item->getUnitPrice()->getNumber() * 100,
      'grossTotalAmount' => $item->getAdjustedTotalPrice()->getNumber() * 100,
      'netTotalAmount' => $item->getTotalPrice()->getNumber() * 100,
    ];

    foreach ($item->getAdjustments() as $adjustment) {
      if ($adjustment->getType() != 'tax') {
        continue;
      }
      $dibsItem['taxAmount'] = $adjustment->getAmount()->getNumber() * 100;
      $dibsItem['taxRate'] = $adjustment->getPercentage() * 100;
    }
    return $dibsItem;
  }

  /**
   * @param \Drupal\commerce_order\Adjustment $adjustment
   *
   * @return array
   */
  public function buildAdjustment(Adjustment $adjustment) {
    return [
      'reference' => $adjustment->getType(),
      'name' => $adjustment->getLabel(),
      'quantity' => 1,
      'unit' => $this->t('Unit'),
      'taxRate' => 0,
      'taxAmount' => 0,
      'unitPrice' => $adjustment->getAmount()->multiply(100)->getNumber(),
      'grossTotalAmount' => $adjustment->getAmount()
        ->multiply(100)
        ->getNumber(),
      'netTotalAmount' => $adjustment->getAmount()
        ->multiply(100)
        ->getNumber(),
    ];
  }
}
